#!/usr/bin/perl -w

BEGIN { push(@INC, "../store/"); };

use WWW::Mechanize;
use WWW::Mechanize::Link;
use Storable;
use db_recipes;

sub parse;
sub parseList;
sub parseListPage;

($name) = $0 =~ m/.*\/(.*)/;

my $INSERT_SPECIALS = 0;

if( $#ARGV == 0 ) {
	$INSERT_SPECIALS = 1;
	parseListPage( "http://allrecipes.com/Search/Recipes.aspx?WithTerm=$ARGV[0]" );
}
elsif( $#ARGV == 1 && $ARGV[0] eq '-u' ) {
	parseListPage( $ARGV[1] );
}
elsif( $#ARGV == 1 && $ARGV[0] eq '-p' ) {
	open( FILE, $ARGV[1] ) or die( "Unable to open $ARGV[1]!\n" );
	@lines = <FILE>;
	parse( \@lines );
	close FILE;
}
else {
	print "Usage: $name [search-string | -u recipe-list-page | -p recipe-page]\n";
	print "	search-string		The string to search allrecipes.com for.\n";
	print "	-u recipe-list-page	The full path and filename to a list of recipe\n";
	print "				pages.\n";
	print "	-p recipe-page		Parse a single recipe page recipe-page.\n";
	die;
}

# parse page(s) of lists of links
sub parseListPage {
	$url = shift;

	$mech = WWW::Mechanize->new( autocheck => 1 );
	$mech->get( $url );

	#create a link to the first page
	$link = WWW::Mechanize::Link->new( { url  => $url } );

	#create a list of other results pages
	@pages = $mech->find_all_links( tag => "a", text_regex => qr/[0-9]/ );

	# parse the links on the first page, since we already have it
	@links = $mech->find_all_links( tag => "a", id_regex => qr/lnkRecipeTitle/i );
	parseList(@links);

	foreach $i (@pages) {

		$link = $i->url;

		$mech->get( $link );

		#find recipe links on the page
		@links = $mech->find_all_links( tag => "a", id_regex => qr/lnkRecipeTitle/i );

		parseList( \@links );
	}
}

# parse a list of links
sub parseList {
	foreach $j (@links) {
		
		# follow the link
		$link = $j->url;
		$mech->get( $link );

		# split into lines
		@lines = split( /\n/, $mech->content );

		parse( \@lines );
	}
}

# parse a single recipe page
sub parse {
	$lines = shift;

	$titleline  = -1;
	$descline   = -1;
	$ingredline = -1;
	$directline = -1;

	# find identifiers
	$count = 0;
	foreach $i (@$lines) {
		# if we found everything stop looking
		if( $titleline > 0 && $descline > 0 && $ingredline > 0 && $directline > 0 ) {last;}

		if( $titleline  < 0 && $i =~ m/<!-- TITLE -->/ )       {$titleline  = $count+2;}
		if( $descline   < 0 && $i =~ m/<!-- DESCRIPTION -->/ ) {$descline   = $count+1;}
		if( $ingredline < 0 && $i =~ m/<!-- INGREDIENTS -->/ ) {$ingredline = $count;}
		if( $directline < 0 && $i =~ m/<!-- DIRECTIONS -->/ )  {$directline = $count;}

		$count++; 
	}

	# pull the title and description
	$title = $lines[$titleline];
	$desc  = $lines[$descline];
	@ingredients = ();
	@directions = ();

	# pull the list of ingredients and trim as we go
	$count = 0;
	for( $i = $ingredline; !($lines[$i] =~ m/<\/ul>/); $i++ ) {
		if( $lines[$i] =~ m/<\/li>/ ) { 			
			$ingredients[$count] = $lines[$i];
			$ingredients[$count] =~ s/^\s+//;
			$ingredients[$count] =~ s/<\/li>\s*\n*$//;
			if ($ingredients[$count] !~/^&nbsp;$/) {
				$count++;
			}
		}
	}

	# pull the list of directions and trim and number as we go
	$count = 1;
	for( $i = $directline; !($lines[$i] =~ m/<\/ul>/); $i++ ) {
		if( $lines[$i] =~ m/<span>/ ) { 
			$directions[$count-1] = $lines[$i+1];
			$directions[$count-1] =~ s/^\s+/$count\. /;
			$directions[$count-1] =~ s/\s*\n*$//;
			$count++;
		}
	}

	# remove unnecessary characters
	$title =~ s/^\s+//;
	$title =~ s/\s+\n*$//;

	$desc =~ s/^\s+"//;
	$desc =~ s/\s*".*\n*$//;

	# print results to output file
	print "$title\n";
	print "$desc\n";

	foreach $i (@ingredients) { print "$i\t"; }
	print "\n";

	foreach $d (@directions) { print "$d\t"; }
	print "\n\n";
	
	if ($INSERT_SPECIALS) {
		InsertRecipe($title, \@ingredients, \@directions);	
	}
}	

