

use strict;
use warnings;
use db_specials;


BEGIN { push(@INC, "../store/"); };

# Routine to insert a recipe into the database
# Database schema for some of the important components 
# recipeingredient
# (recipeId, ingredientId, quantity, preparation)
# recipe
#    (id, name, directions)
# ingredient
#     (id, name)
my $RECIPE_TABLE_NAME = "recipe";
my $INGREDIENT_TABLE_NAME = "ingredient";
my $RECIPE_INGREDIENT_TABLE_NAME = "recipeingredient";
my $INGREDIENT_NAME_MAX_LENGTH = 24;
my $QUANTITY_MAX_LENGTH = 24;
my $PREPARATION_MAX_LENGTH = 24;

sub InsertRecipe {
    my $recipeName = shift;
    my $ingredientsRef = shift;
    my $directionsRef = shift;

    my $recipeId;
    my $dbh = ConnectToDatabase();

    my $rows;
	my $name;
	
	
	$name = GetScalarValue($dbh, $RECIPE_TABLE_NAME, "name", "name = ?", $recipeName);
	
	if (defined($name)) {
		print STDERR "Recipe $name exists, not adding it any more\n";
		$dbh->disconnect();
		return -1;
	}
	
    $dbh->do("INSERT INTO $RECIPE_TABLE_NAME (name, directions) VALUES (?, ?)",
	     undef, $recipeName, join(" ", @$directionsRef));

    $recipeId = GetLastInsertedId($dbh);

    for my $ingredient (@$ingredientsRef) {
		my $trueIngredient;
		my $trueIngredientId;
		my $quantity;
		my $preparation;
		my $query;
		
		$trueIngredient = $ingredient;
		$quantity = $ingredient;
		$preparation = $ingredient;
		
		($trueIngredient, $quantity, $preparation) = parseIngredient($ingredient);
		
		$trueIngredient = substr($trueIngredient, 0, $INGREDIENT_NAME_MAX_LENGTH) if $trueIngredient;
		$quantity         = substr($quantity, 0, $QUANTITY_MAX_LENGTH) if $quantity;
		$preparation    = substr($preparation, 0, $PREPARATION_MAX_LENGTH) if $preparation;
		
		$trueIngredientId = GetScalarValue($dbh, "$INGREDIENT_TABLE_NAME", "id", "name = '$trueIngredient'");	
		
		if (not defined($trueIngredientId)) {
			# Insert ingredient into the database and get the id
			$rows = $dbh->do("INSERT INTO $INGREDIENT_TABLE_NAME (name) VALUES (?)",
									   undef, $trueIngredient);       

			$trueIngredientId = GetLastInsertedId($dbh);
		}

		$query = qq{INSERT INTO $RECIPE_INGREDIENT_TABLE_NAME 
							   (recipeId, ingredientId, quantity, preparation)
							   VALUES 
							   (?, ?, ?, ?)};
		$dbh->do($query, undef, $recipeId, $trueIngredientId, 
					   $quantity, $preparation);
    }

    $dbh->disconnect();
}


# For each ingredient of a recipe
# 
# Insert the ingredient into the database
# Get the ingredient Id
# Parse the ingredient for quantiity and preparation
# Insert the (recipeId, ingredientId, quantity, and preparation) into the
#   recipeingredient

# 

sub TryInsertAndGetId {
    return 0;
}

my $MEASURE_WORDS = "can|cans|cup|slices|envelope|slice|cups|teaspoon|teaspoons|package|jigger|fluid ounces|tablespoon|tablespoons|cans|fluid ounce|ounce|ounces|pound|pounds";
my $PREPARATION_WORDS = "warm|frozen cut|diced|boil|cold";
sub parseIngredient {
	my $falseIngredient = shift;
	my $trueIngredient;
	my $quantity;
	my $preparation;
	open(BAD_FILE, ">>bad_files.txt");
	# Go through a set of keywords that match an ingredient
	if ($falseIngredient =~ m/((\d* )?\d+(\/\d+)?( \(.+\))? ($MEASURE_WORDS)) (.*)/io) {
		$quantity = $1;
		$trueIngredient = $6;
		($trueIngredient, $preparation) = parseTrueIngredient($trueIngredient);
	}
	elsif ($falseIngredient =~ m/(\d+( \(.+\))?) (.*)/) {
		$quantity = $1;
		$preparation = "";
		$trueIngredient = $3;
		($trueIngredient, $preparation) = parseTrueIngredient($trueIngredient);
	}
	else {
		$trueIngredient = $falseIngredient;
		$quantity = undef;
		$preparation = undef;
		print BAD_FILE "Don't know how to parse: $falseIngredient\n";
	}
	close(BAD_FILE);
	
	print "\tFalse Ingredient is $falseIngredient\n";
	print "\tQuantity is $quantity\n" if ($quantity);
	print "\tpreparation is $preparation\n" if ($preparation);
	print "\tTrue Ingredient is $trueIngredient\n";
	return ($trueIngredient, $quantity, $preparation);
}

sub parseTrueIngredient {
	my $trueIngredient = shift;
	my $preparation;
	my $ignoreStrPos;

	($trueIngredient, $preparation) = split(", ", $trueIngredient);
	
	if (not $preparation) {
		if ($trueIngredient =~ /((\w+)?($PREPARATION_WORDS)(w+)?) (.*)/) {
			$trueIngredient = $5;
			$preparation     = $1;
		}
	}
	
	$ignoreStrPos = index( $trueIngredient, "(");

	if ($ignoreStrPos >= 0) {
		$trueIngredient = substr($trueIngredient, 0, $ignoreStrPos - 1);
	}

	return ($trueIngredient, $preparation);
}

sub TestParsingIngredients {
	my $testFile = "parser_test_cases.txt";
	my $line;
	my $state = 0;
	my $falseIngredient, my $expectedQuantity, my $expectedPreparation, my $expectedIngredient;
	open(TEST_CASES, "$testFile") or die("Cannot open $testFile, $@\n");
	while (defined($line = <TEST_CASES>)) {
		if ($line eq "\n") {
			# Ignore blank lines
			;
		}
		else {
			chomp($line);
			$line =~ s/\t//g;
			if ($line =~ /^#/) {
				# Do nothing
				;
			}
			else {
				if ($line =~ /(.*)#/) {
					$line = $1;
				}
				
				if ($state == 0) {
					$falseIngredient = $line;
					$state++;
				}
				elsif ($state == 1) {
					$expectedQuantity = $line;
					$state++;
				}
				elsif ($state == 2) {
					$expectedPreparation = $line;
					$state++;
				}
				elsif ($state == 3) {
					$expectedIngredient = $line;		
					$state = 0;
					print "$falseIngredient\n\t$expectedQuantity\n\t$expectedPreparation\n\t$expectedIngredient\n";	
					my $actualTrueIngredient, my $actualQuantity, my $actualPreparation, my $lastTestStatus;
					($actualTrueIngredient, $actualQuantity, $actualPreparation) = parseIngredient($falseIngredient);
					
					$lastTestStatus = TestString($falseIngredient, $expectedQuantity, $actualQuantity, $lastTestStatus);
					$lastTestStatus = TestString($falseIngredient, $expectedPreparation, $actualPreparation, $lastTestStatus);
					$lastTestStatus = TestString($falseIngredient, $expectedIngredient, $actualTrueIngredient, $lastTestStatus);

				}				
			}
		}
	}

	close(TEST_CASES);
}

sub TestString {
	my $testTitle       = shift;
	my $expectedStr = shift;
	my $actualStr     = shift;
	my $lastTestStatus = shift;
	
	if ($expectedStr ne $actualStr) {
		if ($lastTestStatus >= 0) {
			print STDERR "Test case: ", $testTitle, "\n";
		}
		print STDERR "\tExpected '$expectedStr' got '$actualStr'\n";
		return $lastTestStatus - 1;
	}
	
	return 0;
}	

#TestParsingIngredients();

1;
