#!/bin/perl

# load module
use DBI;
use strict;
use warnings;

my $verbose = 0;
my $dbFile = "../../website/lib/tinymvc/myapp/configs/database.php";
my %dbParams = ();
my $STORE_CHAIN_TABLE_NAME = "storechain";
my $STORE_LOCATION_TABLE_NAME = "storelocation";
my $SPEICAL_TABLE_NAME = "special";


sub PopulateDbParamsHash() {
	my $line;
	open(DB_FILE, "$dbFile") or die("Cannot open $dbFile $@");
	while (defined($line = <DB_FILE>)) {
		chomp($line);
		if ($line =~ /\[\'(\w+)\'\]\s*=\s*\'(\w+)\';/) {
			print $1, " : ", $2, "\n" if ($verbose > 1);
			$dbParams{$1} = $2;
		}
		print $line, "\n" if ($verbose > 2);
	}
	close(DB_FILE);
}

sub ConnectToDatabase() {
    
    # TODO:
    # If the DbParams hash is already populated
    # then don't repopulate it
    PopulateDbParamsHash();
	
    # connect
    my $dbh = DBI->connect("DBI:mysql:database=$dbParams{name};host=$dbParams{host}", 
			   "$dbParams{user}", 
			   "$dbParams{pass}", {'RaiseError' => 1});
    return $dbh;
}

# Go through the StoreLocation
# Get the zip codes
# returns a reference zip codes
# Store chain id
sub GetStoreZipCodes {

    my $storeName = shift;
    my @zipCodeArray = ();
    my $dbh = ConnectToDatabase();

    my $storeChainId = GetScalarValue($dbh, "$STORE_CHAIN_TABLE_NAME", "id", "name = '$storeName'");

    if (defined($storeChainId)) {
	my $sth = $dbh->prepare("SELECT zipCode FROM $STORE_LOCATION_TABLE_NAME WHERE storeChainId = $storeChainId");
	$sth->execute();
	my $row;


	while ($row = $sth->fetchrow_array()) {
	    push(@zipCodeArray, $row);
	}
	$sth->finish();
    }

    $dbh->disconnect();

    return @zipCodeArray;
}



sub TestGetScalarValue() {

    my $dbh = ConnectToDatabase();
    
	# Check for a case when the value exists
    my $storeChainId = GetScalarValue($dbh, "$STORE_CHAIN_TABLE_NAME", "id", "name = 'ALBERTSONS'");
	
	if ($storeChainId && $storeChainId != 1) {
		print STDERR "Failed!!!\n";
		print STDERR "Actual $storeChainId expected 1\n";
	}
	
	print "Is \$storeChainId defined? ", defined($storeChainId) ? "true" : "false","\n";
    print "StoreChainId = $storeChainId\n";
	
    my $storeLocationId =  GetScalarValue($dbh, "$STORE_LOCATION_TABLE_NAME", "id", 
			       "storeChainId = $storeChainId AND zipCode = 98072");
    print "StorelocationId = $storeLocationId\n";
    
}

sub TestZipCodeArray() {
    my @zipCodeArray = GetStoreInformation("QFC");
    print join("\n", @zipCodeArray), "\n";
}

sub InsertSpecialsIntoDb {

	my $storeName = shift;
    my $zipCode = shift;
	my $allSpecialsRef = shift;
	
	my $rows;
	my $storeChainId;
	my $storeLocationId;
	
	# connect
	my $dbh = ConnectToDatabase(); 


	$storeChainId = GetScalarValue($dbh, "$STORE_CHAIN_TABLE_NAME", "id", "name = '$storeName'");

 	$storeLocationId = GetScalarValue($dbh, "$STORE_LOCATION_TABLE_NAME", "id", 
						 "storeChainId = $storeChainId AND zipCode = $zipCode");

	if (defined($storeLocationId)) {
	    
	    my $i = 0;
	    for (my $special = $allSpecialsRef->[$i]; defined($special); $i++, $special = $allSpecialsRef->[$i]) {

		print "Inserting into db: ", $special->[0], " : ", $special->[1], "\n" if ($verbose > 3);
		my $query = qq{INSERT INTO $SPEICAL_TABLE_NAME 
					(storeLocationId, title, price) 
					 VALUES
					(?, ?, ?)};
		$rows = $dbh->do($query, undef, $storeLocationId, $special->[0], $special->[1]);
	    }
	}
	else {
	    print "******No data in perl\n";
	}

	#clean up
	$dbh->disconnect();
}

sub GetLastInsertedId {
	my $dbh = shift;
	my $sth = $dbh->prepare("SELECT LAST_INSERT_ID()");
	$sth->execute();
	my ($lastInsertedId) = $sth->fetchrow_array;
	$sth->finish();
	return $lastInsertedId;
}
	
sub GetScalarValue {
	my $dbh = shift;
	my $tableName = shift;
	my $columnName = shift;
	my $whereCondition = shift;
	my @bind_params = @_;
	
	my $sth = $dbh->prepare("SELECT $columnName FROM $tableName WHERE $whereCondition");
	$sth->execute(@bind_params);
		
	my ($columnValue) = $sth->fetchrow_array;

	$sth->finish();
	return $columnValue;
}

sub Test() {
	my $address = "45th AVE Seattle";
	my $city = "Seattle";
	my $state = "WA";
	my $zipCode = "98105";
	my $phone   = "(206) 999-9999";
	
	my @specials = ( ["cake", "\$20"], 
		  		     ["cake 2", "\$20"],
				     ["cake3", "\$30 per"]
				   );
	
	my $specialRef = \@specials;
	my $special = $specialRef;
	my $i = 0;
	#my $aRow = $special->[$i];
	
	for (my $i = 0, my $aRow = $special->[$i]; defined($aRow); $i++, $aRow = $special->[$i]) {
	
		#print "*************************************\n";
		#print "Items on Page # ", $curPage, "\n";
		#print "*************************************\n";
		print $aRow->[0], " : ", $aRow->[1], "\n";
	}
	
	InsertSpecialsIntoDb($address,
						 $city,
						 $state,
						 $zipCode,
						 $phone,
						 \@specials);
	
	# TODO: More detailed tests here...
}
	
#Test();				  
sub InsertStoreInformation {
    
    my $storeName = shift;
    my $storeWebsite = shift; 
    my $address = shift;
    my $city = shift;
    my $state = shift;
    my $zipCode = shift;
    my $phone   = shift;
	
	
    my $rows;
    my $storeChainId;
    my $storeLocationId;
		
    my $dbh;

    $dbh = ConnectToDatabase();

    # Find if the store already exists in the database, if it does not then insert it into the db
    eval { 
	$rows = $dbh->do("INSERT INTO $STORE_CHAIN_TABLE_NAME (name, websiteUrl) VALUES ('$storeName', '$storeWebsite')");		
    };
	
    if ($@) {
	$storeChainId = GetScalarValue($dbh, "$STORE_CHAIN_TABLE_NAME", "id", "name = '$storeName'");
    }
    else {
	$storeChainId = GetLastInsertedId($dbh);
    }

    # Insert the store location information after making sure another store of the same zip code
    #does not exist
    eval { 
	$rows = $dbh->do(qq{INSERT INTO $STORE_LOCATION_TABLE_NAME 
							(storeChainId, address, city, state, zipCode, phone) 
							VALUES 
							($storeChainId, '$address', '$city', '$state', $zipCode, '$phone')});
    };

    if ($@) {

 	$storeLocationId = GetScalarValue($dbh, "$STORE_LOCATION_TABLE_NAME", "id", 
						 "storeChainId = $storeChainId AND zipCode = $zipCode");
    }

    $dbh->disconnect();
    return ($storeChainId, $storeLocationId);
}

#TestZipCodeArray();

TestGetScalarValue();