#!/usr/local/bin/perl

use strict;
#use warnings;
use WWW::Mechanize;
use HTTP::Cookies;

#use db_specials;
# USAGE:
# perl qfc_new.pl [zipcode]
# If zip code is not specified, 98195 is used

#my $start = "http://albertsons.shoplocal.com/albertsons/Default.aspx?CityStateZip=98195&action=nuep";
my $verbose = 4;

sub InsertSpecialsForZipcode  {

    my $userZipCode = shift;
    my $storeName = "QFC";
    print "Searching under zip code $userZipCode\n";

    my $start = "http://services.qfc.com/mapquest/storelocatorresults.aspx?search3=0&search15=0&search17=0&search19=0&search20=0&Text29=0&Text44=0&Text48=0&Text49=0&Text50=0&Text51=0&Text52=0&radius=20&address=&city=&state=&postalCode=$userZipCode";

    # Albertsons did not work...
    # Perhaps because I did not use any follow_link() functions
    # and used $mech->get() all the time, which stored brand new
    # cookies everytime. Hence, I always got the same empty page about installflash no matter what...
    #my $start = "http://albertsons.shoplocal.com/albertsons/Default.aspx?default.aspx?action=installflash&mode=html";
    #my $start = "http://albertsons.shoplocal.com/albertsons/Default.aspx?action=browsepagesingle&storeid=2407406&rapid=694307&pagenumber=$i&prvid=albertsons-090506-IM";
    #$start = "https://shop.albertsons.com/eCommerceWeb/InOurStoresAction.do?action=showStoreSearch";

    # Create a new Mechanize object with *explicit* cookies enabled
    my $mech = WWW::Mechanize->new(autocheck => 1, cookie_jar => HTTP::Cookies->new);
    
# Not sure if removing my browser agent alias as IE 6 will break things!
    $mech->agent_alias( 'Windows IE 6' );

# Fetch the page
    $mech->get($start);

# Lets do some serious parsing to find the next link
# This code below actually finds the first store (hence the closest one) and 
# "follows" the link to weekly specials
    my $stream = HTML::TokeParser->new(\$mech->{content});
    my $tag = $stream->get_tag("tr");


    while ($tag && not $tag->[1]{"class"} eq "alt") {
	$tag = $stream->get_tag("tr");
    }

# Should have used $verbose everywhere but oh well, later
    if ($verbose > 3) {
	print $tag->[1]{"class"}, "\n";
    }
    $stream->get_tag("td"); # distance
    $stream->get_tag("td"); # qfc
    $stream->get_tag("td"); # Address

    $tag = $stream->get_tag("a");
    my $store = "QFC";
    my $address = $stream->get_trimmed_text("/a");
    my $phone   = $stream->get_trimmed_text("/td");
    
    $stream->get_tag("td");
    my $city    = $stream->get_trimmed_text("/td");

    $stream->get_tag("td");
    my $state   = $stream->get_trimmed_text("/td");
    
    $stream->get_tag("td");
    my $zipCode = $stream->get_trimmed_text("/td");
    
    print "$store $address $phone $city $state $zipCode\n";

# Now get the corresponding Weekly ad link
    $stream->get_tag("td");
    $tag = $stream->get_tag("a");
    my $link = "";
    if ($tag->[1]{"id"} =~ m/WeeklyAd/) {
	$link = $tag->[1]{"href"};
	print "\n", $link, "\n";
    }

# I tried changing customer_Frame to storeLocator here since I thought
# that was a shortcut. But apparently I miss couple of cookies which are 
# important for the pageLarge.jsp to retrieve
#$link =~ s/customer_Frame/storeLocator/;
#$mech->get("http://qfconline.inserts2online.com/pageLarge.jsp?pageNumber=2&drpStoreID=807");
    if ($verbose > 3) {
	print $link, "\n";
    }

# Perhaps follow some links now, will buy me some cookies, yea!!!
    my $newLink = $mech->find_link(url => $link);
    $mech->follow_link(url => $link);
    $mech->follow_link(url_regex => qr/storeReview/);
    $mech->follow_link(text_regex => qr/HTML version/);
    
# I guess I can stop here, now I can go back to using get, lovely
#$mech->follow_link(id => "Large");
#$mech->follow_link(url => "pageLarge.jsp?pageNumber=2&drpStoreId=807&showFlash=false");

# First I need to know how many pages are there
    $mech->get("http://qfconline.inserts2online.com/pageLarge.jsp?pageNumber=1&drpStoreID=807");

    undef($stream);
    $stream = HTML::TokeParser->new(\$mech->{content});

    $tag = $stream->get_tag("a");
    my $pageCount = 0;
    
# I get my page count using the "1:2:3:4..." 'a' link tags
    while (defined($tag)) {
	if ($tag->[1]{"class"} eq "pageNumber") {
	    $pageCount++;
	}
	$tag = $stream->get_tag("a");
    }
    
# Apparently the links are repeated on top and bottom, and 
# the current page does not have a link
    $pageCount = $pageCount/2 + 1;
    print "Number of pages = ", $pageCount, "\n";
    
# Now lets get some specials
    undef($stream);
    $stream = HTML::TokeParser->new(\$mech->{content});

    my $curPage = 1;
    my @allSpecials = ();
    for ($curPage = 1; $curPage <= $pageCount; $curPage++) {

	$mech->get("http://qfconline.inserts2online.com/pageLarge.jsp?pageNumber=$curPage&drpStoreID=807");	
	undef($stream);
	$stream = HTML::TokeParser->new(\$mech->{content});
	
	$tag = $stream->get_tag("div");
	my $price;
	my $item;
	while (defined($tag)) {		
	    if ($tag->[1]{"class"} =~ m/itemName/) {
		$item = $stream->get_trimmed_text("/div");
		$stream->get_tag("div");
		$price = $stream->get_trimmed_text("/div");
		$price =~ s/\xA2/c/;
		$price =~ s/'//g;
		$item  =~ s/'//g;
		
		#print $item, " : ", $price, "\n";
		push(@allSpecials, [($item, $price)]);
	    }
	    $tag = $stream->get_tag("div");
	}
	
    }	
    
    for my $special (@allSpecials) {
	
	#print "*************************************\n";
	#print "Items on Page # ", $curPage, "\n";
	#print "*************************************\n";
	print $special->[0], " : ", $special->[1], "\n";
    }
    
 
    return \@allSpecials;

    # Maybe use zipCode instead?
    # InsertSpecialsIntoDb($storeName,
    #			 $userZipCode,			 
    #			 \@allSpecials);

}

