<?php

/**
 * Session Handler is used to "leave messages" for the next page shown, such as error messages or notices.
 * Session Handler also abstracts the process of setting and getting cookie and session vars.
 * @package Plugins
 */
class Session_Handler {

	/**
	 * Leave a message to be shown by the next page view
	 */
	static function addMessage($message)
	{
		self::start();

		if (!isset($_SESSION['messages'])) {
			$_SESSION['messages'] = array();
		}
		$_SESSION['messages'][] = array(
			'message' => $message,
		);
	}

	/**
	 * Leave an error message, along with a "type" to identify what kind of error it is (i.e. "login")
	 */
	static function addError($error, $type)
	{
		self::start();

		if (!isset($_SESSION['errors'])) {
			$_SESSION['errors'] = array();
		}
		$_SESSION['errors'][] = array(
			'message' => $error,
			'type' => $type,
		);
	}

	/**
	 * @ignore
	 */
	static $_messages;
	/**
	 * @ignore
	 */
	static $_errors;

	/**
	 * Retrieve the messages left in the previous page view 
	 */
	static function getMessages()
	{
		self::start();
		if (self::$_messages)
			return self::$_messages;

		if (isset($_SESSION['messages'])) {
			self::$_messages = $_SESSION['messages'];
			unset($_SESSION['messages']);
			return self::$_messages;
		}

		return array(); // default
	}

	/**
	 * Retrieve the errors left in the previous page view
	 */
	static function getErrors()
	{
		self::start();
		if (self::$_errors)
			return self::$_errors;

		if (isset($_SESSION['errors'])) {
			self::$_errors = $_SESSION['errors'];
			unset($_SESSION['errors']);
			return self::$_errors;
		}

		return array(); // default
	}

	/**
	 * Set a cookie's value.
	 */
	static function setCookie($varname, $value, $expire = 0)
	{
		if (!$varname)
			return false;

		return setcookie($varname, $value, $expire);
	}

	/**
	 * Get a cookie's value.
	 */
	static function getCookie($varname)
	{
		if (isset($_COOKIE[$varname])) {
			return $_COOKIE[$varname];
		} else {
			return null;
		}
	}

	/**
	 * Set a session value
	 */
	static function setSession($varname, $value)
	{
		if ($varname && self::start()) {
			$_SESSION[$varname] = $value;
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Get a session value.
	 */
	static function getSession($varname)
	{
		if ($varname && self::start()) {
			if (isset($_SESSION[$varname])) {
				return $_SESSION[$varname];
			} else {
				return null;
			}
		}
	}

	/**
	 * Start (or resume) the user's session
	 */
	static function start() {
		if (!session_id()) {
			return session_start();
		} else {
			return true;
		}
	}
	
	/**
	 * End the user's session.
	 */
	static function end()
	{
		if (session_id())
			return session_destroy();
		else
			return true;
	}
}

?>